@@ -2,6 +2,15 @@ import tensorflow as tf
 from tensorflow.keras import backend as K
 
 
+def get_feature_sizes(input_shape, pyramid_levels=[3, 7]):
+    # https://github.com/google/automl/tree/master/efficientdet/utils.py#L509
+    feature_sizes = [input_shape[:2]]
+    for _ in range(max(pyramid_levels)):
+        pre_feat_size = feature_sizes[-1]
+        feature_sizes.append(((pre_feat_size[0] - 1) // 2 + 1, (pre_feat_size[1] - 1) // 2 + 1))  # ceil mode, like padding="SAME" downsampling
+    return feature_sizes
+
+
 def get_anchors(input_shape=(512, 512, 3), pyramid_levels=[3, 7], aspect_ratios=[1, 2, 0.5], num_scales=3, anchor_scale=4, grid_zero_start=False):
     """
     >>> from keras_cv_attention_models.coco import anchors_func
@@ -29,12 +38,7 @@ def get_anchors(input_shape=(512, 512, 3), pyramid_levels=[3, 7], aspect_ratios=
 
     # make grid
     pyramid_levels = list(range(min(pyramid_levels), max(pyramid_levels) + 1))
-
-    # https://github.com/google/automl/tree/master/efficientdet/utils.py#L509
-    feature_sizes = [input_shape[:2]]
-    for _ in range(max(pyramid_levels)):
-        pre_feat_size = feature_sizes[-1]
-        feature_sizes.append(((pre_feat_size[0] - 1) // 2 + 1, (pre_feat_size[1] - 1) // 2 + 1))  # ceil mode, like padding="SAME" downsampling
+    feature_sizes = get_feature_sizes(input_shape, pyramid_levels)
 
     all_anchors = []
     for level in pyramid_levels:
@@ -61,6 +65,20 @@ def get_anchor_free_anchors(input_shape=(512, 512, 3), pyramid_levels=[3, 5], gr
     return get_anchors(input_shape, pyramid_levels, aspect_ratios=[1], num_scales=1, anchor_scale=1, grid_zero_start=grid_zero_start)
 
 
+def get_pyramid_levels_by_num_anchors(input_shape, num_anchors, pyramid_levels_min=3):
+    feature_sizes = get_feature_sizes(input_shape, [pyramid_levels_min, pyramid_levels_min + 10])
+    feature_sizes = tf.convert_to_tensor(feature_sizes, dtype="float32")
+    pyramid_levels = []
+    level = pyramid_levels_min
+    while num_anchors > 0:
+        pyramid_levels.append(level)
+        stride_hh, stride_ww = feature_sizes[0][0] / feature_sizes[level][0], feature_sizes[0][1] / feature_sizes[level][1]
+        cur_num_anchors = tf.math.ceil(input_shape[0] / stride_hh) * tf.math.ceil(input_shape[1] / stride_ww)
+        num_anchors -= int(cur_num_anchors)
+        level += 1
+    return pyramid_levels
+
+
 def iou_nd(bboxes, anchors):
     # bboxes: [[top, left, bottom, right]], anchors: [[top, left, bottom, right]]
     anchors_nd, bboxes_nd = tf.expand_dims(anchors, 0), tf.expand_dims(bboxes, 1)
@@ -82,14 +100,13 @@ def corners_to_center_yxhw_nd(ss):
 
 def assign_anchor_classes_by_iou_with_bboxes(bbox_labels, anchors, ignore_threshold=0.4, overlap_threshold=0.5):
     num_anchors = anchors.shape[0]
-    valid_bboxes_pick = tf.where(bbox_labels[:, -1] > 0)[:, 0]
-    bbox_labels = tf.gather(bbox_labels, valid_bboxes_pick)
+    bbox_labels = tf.gather_nd(bbox_labels, tf.where(bbox_labels[:, -1] > 0))
     bboxes, labels = bbox_labels[:, :4], bbox_labels[:, 4]
 
-    anchor_ious = iou_nd(bboxes, anchors)
-    anchor_best_iou_ids = tf.argmax(anchor_ious, axis=0)
+    anchor_ious = iou_nd(bboxes, anchors)  # [num_bboxes, num_anchors]
+    anchor_best_iou_ids = tf.argmax(anchor_ious, axis=0)  # [num_anchors]
     # anchor_best_ious = tf.gather_nd(anchor_ious, tf.stack([anchor_best_iou_ids, tf.range(num_anchors, dtype=anchor_best_iou_ids.dtype)], axis=-1))
-    anchor_best_ious = tf.reduce_max(anchor_ious, axis=0)  # This faster
+    anchor_best_ious = tf.reduce_max(anchor_ious, axis=0)  # This faster, [num_anchors]
 
     matched_idxes = tf.where(anchor_best_ious > overlap_threshold)[:, 0]
     matched_idxes = tf.unique(tf.concat([matched_idxes, tf.argmax(anchor_ious, axis=-1)], axis=0))[0]  # Ensure at leat one anchor selected for each bbox
@@ -134,28 +151,31 @@ def decode_bboxes(preds, anchors):
 
 class AnchorFreeAssignMatching:
     """
+    This has to be after getting model output, as picking matched anchors needs the iou value between prediction and true bboxes.
+
     # Basic test:
     >>> from keras_cv_attention_models.coco import anchors_func
-    >>> anchors = anchors_func.get_anchors([640, 640], pyramid_levels=[3, 5], aspect_ratios=[1], num_scales=1, anchor_scale=1, grid_zero_start=True)
-    >>> aa = anchors_func.AnchorFreeAssignMatching(anchors)
+    >>> aa = anchors_func.AnchorFreeAssignMatching([640, 640])
 
     >>> # Fake data
-    >>> num_bboxes, num_classes = 32, 10
+    >>> num_bboxes, num_classes, num_anchors = 32, 10, 8400
     >>> bboxes_true = tf.random.uniform([num_bboxes, 4], 0, 0.5)
     >>> bboxes_true = tf.concat([bboxes_true[:, :2], bboxes_true[:, 2:] + bboxes_true[:, :2]], axis=-1) # bottom, right > top, left
     >>> labels_true = tf.one_hot(tf.random.uniform([num_bboxes], 0, num_classes, dtype=tf.int32), num_classes)
     >>> valid_bboxes_pick = tf.cast(tf.random.uniform([num_bboxes, 1]) > 0.5, tf.float32)
     >>> bbox_labels_true = tf.concat([bboxes_true, labels_true, valid_bboxes_pick], axis=-1)
-    >>> bbox_labels_pred = tf.random.uniform([aa.num_anchors, 4 + num_classes + 1])
+    >>> bbox_labels_pred = tf.random.uniform([num_anchors, 4 + num_classes + 1])
 
     >>> # Run test
-    >>> bboxes_trues, bboxes_true_encodeds, labels_trues, object_trues, bboxes_preds, bboxes_pred_encodeds, labels_preds = aa(bbox_labels_true, bbox_labels_pred)
-    >>> print(bboxes_trues.shape, bboxes_true_encodeds.shape, labels_trues.shape)
-    >>> # (42, 4), (42, 4), (42, 10)
-    >>> print(bboxes_preds.shape, bboxes_pred_encodeds.shape, labels_preds.shape)
-    >>> # (42, 4), (42, 4), (42, 10)
-    >>> print(f"{object_trues.shape = }, {tf.reduce_sum(object_trues).numpy() = }")
-    >>> # object_trues.shape = TensorShape([4, 8400]), tf.reduce_sum(object_trues).numpy() = 42.0
+    >>> bbox_labels_true_assined = aa(bbox_labels_true, bbox_labels_pred)
+    >>> bboxes_true, bboxes_true_encoded, labels_true, object_true_idx_nd = tf.split(bbox_labels_true_assined, [4, 4, -1, 1], axis=-1)
+    >>> object_true_idx_nd = tf.cast(object_true_idx_nd, tf.int32)
+    >>> object_true_idx = object_true_idx_nd[:, 0]
+    >>> object_true = tf.tensor_scatter_nd_update(tf.zeros_like(bbox_labels_pred[:, -1]), object_true_idx_nd, tf.ones_like(bboxes_true[:, -1]))
+    >>> print(bboxes_true.shape, bboxes_true_encoded.shape, labels_true.shape, tf.reduce_sum(tf.cast(object_true, tf.float32)).numpy())
+    >>> # (42, 4), (42, 4), (42, 10), 42.0
+    >>> print(f"{object_true.shape = }, {bbox_labels_pred[object_true > 0].shape = }")
+    >>> # object_true.shape = TensorShape([8400]), bbox_labels_pred[object_true > 0].shape = TensorShape([42, 15])
 
     # Actual assigning test:
     >>> from keras_cv_attention_models import yolox, test_images
@@ -164,27 +184,32 @@ class AnchorFreeAssignMatching:
     >>> img = test_images.dog_cat()
     >>> pred = mm(mm.preprocess_input(img))
 
-    >>> anchors = anchors_func.get_anchors([640, 640], pyramid_levels=[3, 5], aspect_ratios=[1], num_scales=1, anchor_scale=1, grid_zero_start=True)
-    >>> aa = anchors_func.AnchorFreeAssignMatching(anchors)
+    >>> aa = anchors_func.AnchorFreeAssignMatching([640, 640])
     >>> bbs, lls, ccs = mm.decode_predictions(pred)[0]
     >>> bbox_labels_true = tf.concat([bbs, tf.one_hot(lls, 80), tf.ones([bbs.shape[0], 1])], axis=-1)
-    >>> bboxes_trues, bboxes_true_encodeds, labels_trues, object_trues, bboxes_preds, bboxes_pred_encodeds, labels_preds = aa(bbox_labels_true, pred[0])
-    >>> data.show_image_with_bboxes(img, bboxes_preds, labels_preds.numpy().argmax(-1), labels_preds.numpy().max(-1))
+    >>> bbox_labels_true_assined = aa(bbox_labels_true, pred[0])
+    >>> bboxes_true, bboxes_true_encoded, labels_true, object_true_idx_nd = tf.split(bbox_labels_true_assined, [4, 4, -1, 1], axis=-1)
+    >>> object_true_idx_nd = tf.cast(object_true_idx_nd, tf.int32)
+    >>> object_true_idx = object_true_idx_nd[:, 0]
+    >>> object_true = tf.tensor_scatter_nd_update(tf.zeros_like(pred[0, :, -1]), object_true_idx_nd, tf.ones_like(bboxes_true[:, -1]))
+    >>> # Decode predictions
+    >>> bbox_labels_pred_valid, anchors = pred[0][object_true > 0], aa.anchors[object_true > 0]
+    >>> bboxes_preds, labels_preds, object_preds = bbox_labels_pred_valid[:, :4], bbox_labels_pred_valid[:, 4:-1], pred[0][:, -1:]
+    >>> bboxes_preds_decode = anchors_func.decode_bboxes(bboxes_preds, anchors)
+    >>> # Show matched predictions
+    >>> data.show_image_with_bboxes(img, bboxes_preds_decode, labels_preds.numpy().argmax(-1), labels_preds.numpy().max(-1))
+    >>> # Show gathered bbox ground truth
+    >>> data.show_image_with_bboxes(img, bboxes_true, labels_true.numpy().argmax(-1), labels_true.numpy().max(-1))
+    >>> # Show gathered encoded, bbox ground truth
+    >>> bboxes_true_decode = anchors_func.decode_bboxes(bboxes_true_encoded, anchors)
+    >>> data.show_image_with_bboxes(img, bboxes_true_decode, labels_true.numpy().argmax(-1), labels_true.numpy().max(-1))
     """
 
-    # def __init__(self, input_shape=(640, 640, 3), pyramid_levels=[3, 5], grid_zero_start=True, center_radius=2.5, topk_ious_max=10, epsilon=1e-8):
-    #     self.anchors = get_anchors(input_shape, pyramid_levels, aspect_ratios=[1], num_scales=1, anchor_scale=1, grid_zero_start=grid_zero_start)
-    def __init__(self, anchors, center_radius=2.5, topk_ious_max=10, epsilon=1e-8):
-        self.anchors = anchors
-        self.num_anchors = self.anchors.shape[0]
+    def __init__(self, input_shape, pyramid_levels=[3, 5], center_radius=2.5, topk_ious_max=10, grid_zero_start=True, epsilon=1e-8):
         self.center_radius, self.topk_ious_max, self.epsilon = center_radius, topk_ious_max, epsilon
-        self.is_built = False
-
-    def __build__(self, true_input_shape):
-        self.num_classes = true_input_shape[-1] - 4 - 1
-        self.empty_bboxes = tf.zeros([0, 4])
-        self.empty_labels = tf.zeros([0, self.num_classes])
-        self.empty_objects = tf.zeros([self.num_anchors, 1])
+        self.input_shape, self.grid_zero_start = input_shape, grid_zero_start
+        # pyramid_levels = get_pyramid_levels_by_num_anchors(self.input_shape, self.num_anchors)
+        self.anchors = get_anchors(self.input_shape, pyramid_levels, aspect_ratios=[1], num_scales=1, anchor_scale=1, grid_zero_start=self.grid_zero_start)
 
         # Anchors constant values
         self.anchors_centers = (self.anchors[:, :2] + self.anchors[:, 2:]) * 0.5
@@ -193,8 +218,6 @@ class AnchorFreeAssignMatching:
         self.anchors_centers_nd, self.anchors_hws_nd = tf.expand_dims(self.anchors_centers, 0), tf.expand_dims(self.anchors_hws, 0)  # [1, num_anchors, 2]
         self.centers_enlarge_nd = self.anchors_hws_nd * self.center_radius
 
-        self.is_built = True
-
     def __picking_anchors_by_center_within_bboxes__(self, bboxes_true_nd):
         # get_in_boxes_info https://github.com/Megvii-BaseDetection/YOLOX/tree/master/yolox/models/yolo_head.py#L522
         # bboxes: [[top, left, bottom, right]], anchors: [[top, left, bottom, right]]
@@ -263,13 +286,9 @@ class AnchorFreeAssignMatching:
             lambda: topk_anchors,
         )
 
-    # @tf.function
     def __call__(self, bbox_labels_true, bbox_labels_pred):
-        if not self.is_built:
-            self.__build__(bbox_labels_true.shape)
         # get_assignments https://github.com/Megvii-BaseDetection/YOLOX/tree/master/yolox/models/yolo_head.py#425
-        valid_bboxes_pick = tf.where(bbox_labels_true[:, -1] > 0)[:, 0]
-        bbox_labels_true = tf.gather(bbox_labels_true, valid_bboxes_pick)
+        bbox_labels_true = tf.gather_nd(bbox_labels_true, tf.where(bbox_labels_true[:, -1] > 0))
 
         bboxes_true, labels_true = bbox_labels_true[:, :4], bbox_labels_true[:, 4:-1]
         bboxes_true_nd = tf.expand_dims(bboxes_true, 1)
@@ -279,6 +298,7 @@ class AnchorFreeAssignMatching:
         # [num_anchors]
         is_anchor_match_any_bbox = tf.logical_or(tf.reduce_any(is_anchor_in_bbox, axis=0), tf.reduce_any(is_anchor_in_center, axis=0))
         pick_cond = tf.where(is_anchor_match_any_bbox)[:, 0]
+        # [num_bboxes, num_picked_anchors]
         is_anchor_valid = tf.logical_and(tf.gather(is_anchor_in_bbox, pick_cond, axis=-1), tf.gather(is_anchor_in_center, pick_cond, axis=-1))
 
         bbox_labels_pred = bbox_labels_pred[is_anchor_match_any_bbox]
@@ -287,7 +307,6 @@ class AnchorFreeAssignMatching:
         # decode_bboxes
         anchors_centers, anchors_hws = self.anchors_centers[is_anchor_match_any_bbox], self.anchors_hws[is_anchor_match_any_bbox]
         bboxes_pred_top_left, bboxes_pred_bottom_right, bboxes_pred_center, bboxes_pred_hw = self.__decode_bboxes__(bboxes_pred, anchors_centers, anchors_hws)
-        bboxes_pred_decoded = tf.concat([bboxes_pred_top_left, bboxes_pred_bottom_right], axis=-1)  # still output [top, left, bottom, right]
 
         ious = self.__center_iou_nd__(bboxes_true_nd, bboxes_pred_top_left, bboxes_pred_bottom_right, bboxes_pred_hw)  # [num_bboxes, num_picked_anchors]
         ious_loss = -tf.math.log(ious + self.epsilon)
@@ -300,33 +319,25 @@ class AnchorFreeAssignMatching:
         # dynamic_k_matching
         bbox_matched_k_anchors = self.__dynamic_k_matching__(ious, cost)  # [num_bboxes, num_picked_anchors], contains only 0, 1
         is_anchor_iou_match_any = tf.reduce_any(bbox_matched_k_anchors > 0, axis=0)  # [num_picked_anchors]
-        is_anchor_iou_match_any_idx = tf.where(is_anchor_iou_match_any)[:, 0]
+        is_anchor_iou_match_any_idx = tf.where(is_anchor_iou_match_any)
 
         # TODO: is_anchor_iou_match_any_idx.shape[0] == 0
-        anchor_best_matching_bbox = tf.argmax(tf.gather(bbox_matched_k_anchors, is_anchor_iou_match_any_idx, axis=-1), 0)
+        anchor_best_matching_bbox = tf.argmax(tf.gather(bbox_matched_k_anchors, is_anchor_iou_match_any_idx[:, 0], axis=-1), 0)
         anchor_labels = tf.gather(labels_true, anchor_best_matching_bbox)  # [num_picked_anchors， num_classes]
         pred_iou_loss = tf.reduce_sum(bbox_matched_k_anchors * ious, 0)[is_anchor_iou_match_any]
 
         # get_losses after get_assignments. Bboxes for iou loss, [top, left, bottom, right]
         out_bboxes_true = tf.gather(bboxes_true, anchor_best_matching_bbox)
-        out_bboxes_pred = tf.gather(bboxes_pred_decoded, is_anchor_iou_match_any_idx)
         out_labels_true = anchor_labels * tf.expand_dims(pred_iou_loss, -1)
-        out_labels_pred = tf.gather(labels_pred, is_anchor_iou_match_any_idx)
 
         # object loss, [num_anchors]
-        is_anchor_in_bbox_or_center_valid = tf.tensor_scatter_nd_update(is_anchor_match_any_bbox, tf.where(is_anchor_match_any_bbox), is_anchor_iou_match_any)
-        out_object_true = tf.cast(is_anchor_in_bbox_or_center_valid, out_labels_true.dtype)
-        # out_object_true = tf.expand_dims(out_object_true, -1)
+        out_object_true = tf.tensor_scatter_nd_update(is_anchor_match_any_bbox, tf.where(is_anchor_match_any_bbox), is_anchor_iou_match_any)
+        object_true_idx = tf.where(out_object_true)  # [num_picked_anchors, 1]
 
         # l1_target loss, encoded [center_top, center_left, height, width]
-        anchors_centers_valid, anchors_hws_valid = tf.gather(anchors_centers, is_anchor_iou_match_any_idx), tf.gather(anchors_hws, is_anchor_iou_match_any_idx)
+        anchors_centers_valid = tf.gather_nd(anchors_centers, is_anchor_iou_match_any_idx)
+        anchors_hws_valid = tf.gather_nd(anchors_hws, is_anchor_iou_match_any_idx)
         out_bboxes_true_encoded = self.__encode_bboxes__(out_bboxes_true, anchors_centers_valid, anchors_hws_valid)
-        # out_bboxes_true_encoded = tf.gather(bboxes_true_encoded, anchor_best_matching_bbox)
-        out_bboxes_pred_encoded = tf.gather(bboxes_pred, is_anchor_iou_match_any_idx)
 
-        return out_bboxes_true, out_bboxes_true_encoded, out_labels_true, out_object_true, out_bboxes_pred, out_bboxes_pred_encoded, out_labels_pred
-
-    @tf.function
-    def tf_call(self, bbox_labels_true, bbox_labels_pred):
-        # Have to decorate with tf.function, or will throw error for using in loss function
-        return self.__call__(bbox_labels_true, bbox_labels_pred)
+        # tf.stop_gradient requires returning value been a single tensor with same dtype as inputs.
+        return tf.concat([out_bboxes_true, out_bboxes_true_encoded, out_labels_true, tf.cast(object_true_idx, out_bboxes_true.dtype)], axis=-1)
