@@ -314,7 +314,7 @@ def init_dataset(
     anchor_scale=4,  # Force using 1 if use_anchor_free_mode
     anchor_grid_zero_start="auto",  # False for anchor_free_mode, True for others.
     rescale_mode="torch",  # rescale mode, ["tf", "torch"], or specific `(mean, std)` like `(128.0, 128.0)`
-    random_crop_mode=1.0,
+    random_crop_mode=1.0, # 0 for eval mode, (0, 1) for random crop, 1 for random largest crop, > 1 for random scale
     mosaic_mix_prob=0.0,
     resize_method="bilinear",  # ["bilinear", "bicubic"]
     resize_antialias=False,
@@ -358,10 +358,6 @@ def init_dataset(
     rescaling = lambda xx: (xx - mean) / std
     if use_anchor_free_mode:
         bbox_process = lambda bb: to_one_hot_with_class_mark(tf.concat([bb[0], tf.cast(tf.expand_dims(bb[1], -1), bb[0].dtype)], axis=-1), num_classes)
-        # Init anchors here, or will need these anchor params else where in loss / anchor_assign functions
-        aspect_ratios, num_scales, anchor_scale = [1], 1, 1
-        grid_zero_start = False if anchor_grid_zero_start == "auto" else anchor_grid_zero_start
-        anchors = anchors_func.get_anchors(input_shape[:2], anchor_pyramid_levels, aspect_ratios, num_scales, anchor_scale, grid_zero_start)
     else:
         grid_zero_start = True if anchor_grid_zero_start == "auto" else anchor_grid_zero_start
         anchors = anchors_func.get_anchors(input_shape[:2], anchor_pyramid_levels, anchor_aspect_ratios, anchor_num_scales, anchor_scale, grid_zero_start)
@@ -380,7 +376,7 @@ def init_dataset(
         test_dataset = test_dataset.map(test_process).batch(batch_size).map(lambda xx, yy: (rescaling(xx), bbox_process(yy)))
 
     # Also returns anchors, it's needed for anchor_freee mode
-    return train_dataset, test_dataset, total_images, num_classes, steps_per_epoch, anchors
+    return train_dataset, test_dataset, total_images, num_classes, steps_per_epoch
 
 
 """ Show """
