@@ -117,7 +117,7 @@ def CSPDarknet(width_mul=1, depth_mul=1, out_features=[-3, -2, -1], use_depthwis
 """ path aggregation fpn """
 
 
-def upsample_merge(inputs, csp_depth_mul, use_depthwise_conv=False, activation="swish", name=""):
+def upsample_merge(inputs, csp_depth, use_depthwise_conv=False, activation="swish", name=""):
     # print(f">>>> upsample_merge inputs: {[ii.shape for ii in inputs] = }")
     target_channel = inputs[-1].shape[-1]
     fpn_out = conv_dw_pw_block(inputs[0], target_channel, activation=activation, name=name + "fpn_")
@@ -125,15 +125,15 @@ def upsample_merge(inputs, csp_depth_mul, use_depthwise_conv=False, activation="
     # inputs[0] = keras.layers.UpSampling2D(size=(2, 2), interpolation="nearest", name=name + "up")(fpn_out)
     inputs[0] = tf.image.resize(fpn_out, tf.shape(inputs[-1])[1:-1], method="nearest")
     nn = tf.concat(inputs, axis=-1)
-    nn = csp_stack(nn, round(3 * csp_depth_mul), target_channel, 0.5, False, use_depthwise_conv, activation=activation, name=name)
+    nn = csp_stack(nn, csp_depth, target_channel, 0.5, False, use_depthwise_conv, activation=activation, name=name)
     return fpn_out, nn
 
 
-def downsample_merge(inputs, csp_depth_mul, use_depthwise_conv=False, activation="swish", name=""):
+def downsample_merge(inputs, csp_depth, use_depthwise_conv=False, activation="swish", name=""):
     # print(f">>>> downsample_merge inputs: {[ii.shape for ii in inputs] = }")
     inputs[0] = conv_dw_pw_block(inputs[0], inputs[-1].shape[-1], 3, 2, use_depthwise_conv, activation=activation, name=name + "down_")
     nn = tf.concat(inputs, axis=-1)
-    nn = csp_stack(nn, round(3 * csp_depth_mul), nn.shape[-1], 0.5, False, use_depthwise_conv, activation=activation, name=name)
+    nn = csp_stack(nn, csp_depth, nn.shape[-1], 0.5, False, use_depthwise_conv, activation=activation, name=name)
     return nn
 
 
@@ -143,15 +143,16 @@ def path_aggregation_fpn(features, depth_mul=1, use_depthwise_conv=False, activa
     # x1 ─> f_out0 ─> fpn_out1 ─> pan_out1
     #                    ↓           ↑
     # x2 ───────────> pan_out2 ──────┘
+    csp_depth = max(round(depth_mul * 3), 1)
     [x2, x1, x0] = features  # x2: [64, 64, 256], x1: [32, 32, 512], x0: [16, 16, 1024]
     # fpn_out0: [16, 16, 512], f_out0: [32, 32, 512]
-    fpn_out0, f_out0 = upsample_merge([x0, x1], depth_mul, use_depthwise_conv=use_depthwise_conv, activation=activation, name=name + "c3p4_")
+    fpn_out0, f_out0 = upsample_merge([x0, x1], csp_depth, use_depthwise_conv=use_depthwise_conv, activation=activation, name=name + "c3p4_")
     # fpn_out1: [32, 32, 256], pan_out2: [64, 64, 256]
-    fpn_out1, pan_out2 = upsample_merge([f_out0, x2], depth_mul, use_depthwise_conv=use_depthwise_conv, activation=activation, name=name + "c3p3_")
+    fpn_out1, pan_out2 = upsample_merge([f_out0, x2], csp_depth, use_depthwise_conv=use_depthwise_conv, activation=activation, name=name + "c3p3_")
     # pan_out1: [32, 32, 512]
-    pan_out1 = downsample_merge([pan_out2, fpn_out1], depth_mul, use_depthwise_conv=use_depthwise_conv, activation=activation, name=name + "c3n3_")
+    pan_out1 = downsample_merge([pan_out2, fpn_out1], csp_depth, use_depthwise_conv=use_depthwise_conv, activation=activation, name=name + "c3n3_")
     # pan_out0: [16, 16, 1024]
-    pan_out0 = downsample_merge([pan_out1, fpn_out0], depth_mul, use_depthwise_conv=use_depthwise_conv, activation=activation, name=name + "c3n4_")
+    pan_out0 = downsample_merge([pan_out1, fpn_out0], csp_depth, use_depthwise_conv=use_depthwise_conv, activation=activation, name=name + "c3n4_")
     return [pan_out2, pan_out1, pan_out0]
 
 
@@ -168,30 +169,34 @@ def yolox_head_single(inputs, out_channels, num_classes=80, num_anchors=1, use_d
     cls_nn = conv_dw_pw_block(stem, out_channels, kernel_size=3, use_depthwise_conv=use_depthwise_conv, activation=activation, name=name + "cls_1_")
     cls_nn = conv_dw_pw_block(cls_nn, out_channels, kernel_size=3, use_depthwise_conv=use_depthwise_conv, activation=activation, name=name + "cls_2_")
     cls_out = keras.layers.Conv2D(num_classes * num_anchors, kernel_size=1, bias_initializer=bias_init, name=name + "class_out")(cls_nn)
-    cls_out = activation_by_name(cls_out, "sigmoid", name=name + "class_out")
+    cls_out = activation_by_name(cls_out, "sigmoid", name=name + "class_out_")
+    cls_out = keras.layers.Reshape([-1, num_classes], name=name + "class_out_reshape")(cls_out)
 
     # reg_convs, reg_preds
     reg_nn = conv_dw_pw_block(stem, out_channels, kernel_size=3, use_depthwise_conv=use_depthwise_conv, activation=activation, name=name + "reg_1_")
     reg_nn = conv_dw_pw_block(reg_nn, out_channels, kernel_size=3, use_depthwise_conv=use_depthwise_conv, activation=activation, name=name + "reg_2_")
     reg_out = keras.layers.Conv2D(4 * num_anchors, kernel_size=1, name=name + "regression_out")(reg_nn)
+    reg_out = keras.layers.Reshape([-1, 4], name=name + "regression_out_reshape")(reg_out)
 
     # obj_preds
     if use_object_scores:
         obj_out = keras.layers.Conv2D(1 * num_anchors, kernel_size=1, bias_initializer=bias_init, name=name + "object_out")(reg_nn)
-        obj_out = activation_by_name(obj_out, "sigmoid", name=name + "object_out")
+        obj_out = activation_by_name(obj_out, "sigmoid", name=name + "object_out_")
+        obj_out = keras.layers.Reshape([-1, 1], name=name + "object_out_reshape")(obj_out)
         return tf.concat([reg_out, cls_out, obj_out], axis=-1)
     else:
         return tf.concat([reg_out, cls_out], axis=-1)
 
 
-def YOLOXHead(inputs, width_mul=1.0, num_classes=80, num_anchors=1, use_depthwise_conv=False, use_object_scores=True, activation="swish", name=""):
+def yolox_head(inputs, width_mul=1.0, num_classes=80, num_anchors=1, use_depthwise_conv=False, use_object_scores=True, activation="swish", name=""):
     out_channel = int(256 * width_mul)
     outputs = []
     for id, input in enumerate(inputs):
         cur_name = name + "{}_".format(id + 1)
         out = yolox_head_single(input, out_channel, num_classes, num_anchors, use_depthwise_conv, use_object_scores, activation=activation, name=cur_name)
         outputs.append(out)
-    outputs = tf.concat([keras.layers.Reshape([-1, ii.shape[-1]])(ii) for ii in outputs], axis=1)
+    # outputs = tf.concat([keras.layers.Reshape([-1, ii.shape[-1]])(ii) for ii in outputs], axis=1)
+    outputs = tf.concat(outputs, axis=1)
     return outputs
 
 
@@ -204,23 +209,22 @@ def YOLOX(
     depth_mul=1,
     width_mul=-1,  # -1 means: `min([ii.shape[-1] for ii in features]) / 256` for custom backbones.
     use_depthwise_conv=False,
-    use_object_scores=True,
-    num_anchors=1,
+    use_anchor_free_mode=True,
+    num_anchors="auto",  # "auto" means 1 if use_anchor_free_mode else 9
+    use_object_scores="auto",  # "auto" means same with use_anchor_free_mode
     input_shape=(640, 640, 3),
     num_classes=80,
     activation="swish",
     freeze_backbone=False,
     pretrained=None,
     model_name="yolox",
-    pyramid_levels=[3, 5],  # Init anchors for model prediction.
-    anchor_aspect_ratios=[1],  # Init anchors for model prediction
-    anchor_num_scales=1,  # Init anchors for model prediction
-    anchor_scale=1,  # Init anchors for model prediction.
-    anchor_grid_zero_start=True,  # Init anchors for model prediction
+    pyramid_levels_min=3,  # Init anchors for model prediction.
+    anchor_scale="auto",  # Init anchors for model prediction. "auto" means 1 if use_anchor_free_mode else 4
     rescale_mode="raw",  # For decode predictions, raw means input value in range [0, 255].
     kwargs=None,  # Not using, recieving parameter
 ):
     if backbone is None:
+        width_mul = width_mul if width_mul > 0 else 1
         backbone = CSPDarknet(width_mul, depth_mul, features_pick, use_depthwise_conv, input_shape, activation=activation, model_name="darknet")
         features = backbone.outputs
     else:
@@ -240,37 +244,41 @@ def YOLOX(
         backbone.trainable = True
 
     inputs = backbone.inputs[0]
+    use_object_scores = use_anchor_free_mode if use_object_scores == "auto" else use_object_scores
+    num_anchors = (1 if use_anchor_free_mode else 9) if num_anchors == "auto" else num_anchors
     fpn_features = path_aggregation_fpn(features, depth_mul=depth_mul, use_depthwise_conv=use_depthwise_conv, activation=activation, name="pafpn_")
-    outputs = YOLOXHead(fpn_features, width_mul, num_classes, num_anchors, use_depthwise_conv, use_object_scores, activation=activation, name="head_")
+    outputs = yolox_head(fpn_features, width_mul, num_classes, num_anchors, use_depthwise_conv, use_object_scores, activation=activation, name="head_")
     outputs = keras.layers.Activation("linear", dtype="float32", name="outputs_fp32")(outputs)
     model = keras.models.Model(inputs, outputs, name=model_name)
     reload_model_weights(model, PRETRAINED_DICT, "yolox", pretrained)
 
-    ANCHORS = {"aspect_ratios": anchor_aspect_ratios, "num_scales": anchor_num_scales, "anchor_scale": anchor_scale, "grid_zero_start": anchor_grid_zero_start}
-    post_process = DecodePredictions(backbone.input_shape[1:], pyramid_levels, **ANCHORS, use_object_scores=use_object_scores)
+    # AA = {"aspect_ratios": anchor_aspect_ratios, "num_scales": anchor_num_scales, "anchor_scale": anchor_scale, "grid_zero_start": anchor_grid_zero_start}
+    pyramid_levels = [pyramid_levels_min, pyramid_levels_min + len(features_pick) - 1]  # -> [3, 5]
+    anchor_scale = (1 if use_anchor_free_mode else 4) if anchor_scale == "auto" else anchor_scale
+    post_process = DecodePredictions(backbone.input_shape[1:], pyramid_levels, anchor_scale, use_anchor_free_mode, use_object_scores)
     add_pre_post_process(model, rescale_mode=rescale_mode, post_process=post_process)
     return model
 
 
 def YOLOXNano(input_shape=(416, 416, 3), freeze_backbone=False, num_classes=80, backbone=None, activation="swish", pretrained="coco", **kwargs):
-    return YOLOX(**locals(), depth_mul=0.33, width_mul=0.25, use_depthwise_conv=True, model_name="yolox_nano", **kwargs)
+    return YOLOX(**locals(), depth_mul=0.33, width_mul=0.25, use_depthwise_conv=True, model_name=kwargs.pop("model_name", "yolox_nano"), **kwargs)
 
 
 def YOLOXTiny(input_shape=(416, 416, 3), freeze_backbone=False, num_classes=80, backbone=None, activation="swish", pretrained="coco", **kwargs):
-    return YOLOX(**locals(), depth_mul=0.33, width_mul=0.375, model_name="yolox_tiny", **kwargs)
+    return YOLOX(**locals(), depth_mul=0.33, width_mul=0.375, model_name=kwargs.pop("model_name", "yolox_tiny"), **kwargs)
 
 
 def YOLOXS(input_shape=(640, 640, 3), freeze_backbone=False, num_classes=80, backbone=None, activation="swish", pretrained="coco", **kwargs):
-    return YOLOX(**locals(), depth_mul=0.33, width_mul=0.5, model_name="yolox_s", **kwargs)
+    return YOLOX(**locals(), depth_mul=0.33, width_mul=0.5, model_name=kwargs.pop("model_name", "yolox_s"), **kwargs)
 
 
 def YOLOXM(input_shape=(640, 640, 3), freeze_backbone=False, num_classes=80, backbone=None, activation="swish", pretrained="coco", **kwargs):
-    return YOLOX(**locals(), depth_mul=0.67, width_mul=0.75, model_name="yolox_m", **kwargs)
+    return YOLOX(**locals(), depth_mul=0.67, width_mul=0.75, model_name=kwargs.pop("model_name", "yolox_m"), **kwargs)
 
 
 def YOLOXL(input_shape=(640, 640, 3), freeze_backbone=False, num_classes=80, backbone=None, activation="swish", pretrained="coco", **kwargs):
-    return YOLOX(**locals(), depth_mul=1.0, width_mul=1.0, model_name="yolox_l", **kwargs)
+    return YOLOX(**locals(), depth_mul=1.0, width_mul=1.0, model_name=kwargs.pop("model_name", "yolox_l"), **kwargs)
 
 
 def YOLOXX(input_shape=(640, 640, 3), freeze_backbone=False, num_classes=80, backbone=None, activation="swish", pretrained="coco", **kwargs):
-    return YOLOX(**locals(), depth_mul=1.33, width_mul=1.25, model_name="yolox_x", **kwargs)
+    return YOLOX(**locals(), depth_mul=1.33, width_mul=1.25, model_name=kwargs.pop("model_name", "yolox_x"), **kwargs)
